==> fluid_sbom-1.1.11.tar.gz: fluid_sbom-1.1.11/fluid_sbom/cli/__init__.py <==
# pylint:disable=no-value-for-parameter
import click
from concurrent.futures import (
    ThreadPoolExecutor,
)
from fluid_sbom.advisories.database import (
    DATABASE,
)
from fluid_sbom.cli.options import (
    MutuallyExclusiveOption,
    RequiredAnyCommand,
)
from fluid_sbom.cli.sbom_config import (
    build_sbom_config,
    ScanArgs,
)
from fluid_sbom.config.bugsnag import (
    initialize_bugsnag,
)
from fluid_sbom.config.config import (
    SourceType,
)
from fluid_sbom.config.logger import (
    configure_logger,
)
from fluid_sbom.format import (
    format_sbom,
)
from fluid_sbom.internal.file_resolver.container_image import (
    ContainerImage,
    ImageContext,
)
from fluid_sbom.pkg.cataloger.complete import (
    complete_package,
)
from fluid_sbom.pkg.operations.package_operation import (
    package_operations_factory,
)
from fluid_sbom.sources.directory_source import (
    Directory,
)
from fluid_sbom.sources.docker import (
    extract_docker_image,
    get_docker_image,
    ImageMetadata,
)
import logging
import os
import tempfile
import textwrap
from typing import (
    Unpack,
)

LOGGER = logging.getLogger(__name__)


def show_banner() -> None:
    logo = textwrap.dedent(
        """
         ───── ⌝
        |    ⌝|  Fluid Attacks
        |  ⌝  |  We hack your software.
         ─────
        """
    )
    click.secho(logo, fg="red")


def get_context(
    image: ImageMetadata,
    *,
    username: str | None = None,
    password: str | None = None,
    token: str | None = None,
    daemon: bool = False,
) -> ImageContext | None:
    temp_dir = tempfile.mkdtemp()
    layers_dir, manifest = extract_docker_image(
        image,
        temp_dir,
        username=username,
        password=password,
        token=token,
        daemon=daemon,
    )

    return ImageContext(
        id=image.digest,
        name=image.name,
        publisher="",
        arch=image.architecture,
        size=str(sum(x.size for x in image.layersdata)),
        full_extraction_dir=temp_dir,
        layers_dir=layers_dir,
        manifest=manifest,
        image_ref=image.image_ref,
    )


@click.command(cls=RequiredAnyCommand, required_any=["o_from", "config"])
@click.argument("arg")
@click.option(
    "--from",
    "o_from",
    type=click.Choice(
        ["docker", "dir", "docker-daemon"], case_sensitive=False
    ),
    help=(
        "Source of the scan: 'docker' for scanning Docker images "
        "or 'dir' for scanning directories."
    ),
    cls=MutuallyExclusiveOption,
    mutually_exclusive=["config"],
)
@click.option(
    "--format",
    "output_format",
    type=click.Choice(
        [
            "fluid-json",
            "cyclonedx-json",
            "spdx-json",
            "cyclonedx-xml",
            "spdx-xml",
        ],
        case_sensitive=False,
    ),
    default="fluid-json",
    help="Output format for the scanned data.",
    cls=MutuallyExclusiveOption,
    mutually_exclusive=["config"],
)
@click.option(
    "--output",
    "-o",
    help="Output filename.",
    default="sbom",
    cls=MutuallyExclusiveOption,
    mutually_exclusive=["config"],
)
@click.option(
    "--docker-user",
    default=None,
    help="Docker registry username.",
    cls=MutuallyExclusiveOption,
    mutually_exclusive=["config"],
)
@click.option(
    "--docker-password",
    default=None,
    help="Docker registry password.",
    cls=MutuallyExclusiveOption,
    mutually_exclusive=["config"],
)
@click.option(
    "--config",
    "-c",
    is_flag=True,
    default=False,
    cls=MutuallyExclusiveOption,
    help="Path to an advanced configuration file with additional settings.",
    mutually_exclusive=[
        "docker-user",
        "docker-password",
        "output_format",
        "output",
        "o_from",
    ],
)
def scan(arg: str, **kwargs: Unpack[ScanArgs]) -> None:
    configure_logger()
    initialize_bugsnag()
    show_banner()

    sbom_config = build_sbom_config(arg, **kwargs)

    match sbom_config.source_type:
        case SourceType.DIRECTORY:
            sbom_config.resolver = Directory(
                root=sbom_config.source, exclude=sbom_config.exclude
            )
        case SourceType.DOCKER | SourceType.DOCKER_DAEMON:
            daemon = sbom_config.source_type == SourceType.DOCKER_DAEMON
            docker_image = get_docker_image(
                sbom_config.source,
                username=sbom_config.docker_user,
                password=sbom_config.docker_password,
                daemon=daemon,
            )
            if not docker_image:
                raise ValueError(f"No image found for {sbom_config.source}")

            context = get_context(
                docker_image,
                username=sbom_config.docker_user,
                password=sbom_config.docker_password,
                daemon=daemon,
            )
            if context is None:
                raise ValueError(f"No context found for {docker_image}")
            sbom_config.resolver = ContainerImage(
                img=docker_image, context=context, lazy=False
            )
        case _:
            raise ValueError(f"Unknown source: {sbom_config.source}")

    DATABASE.initialize()
    LOGGER.info(
        "📦 Generating SBOM from %s: %s",
        sbom_config.source_type.value,
        sbom_config.source,
    )

    packages, relationships = package_operations_factory(sbom_config.resolver)
    with ThreadPoolExecutor(
        max_workers=min(
            32, (os.cpu_count() or 1) * 5 if os.cpu_count() is not None else 32
        )
    ) as executor:
        LOGGER.info("📦 Gathering additional package information")
        packages = list(filter(None, executor.map(complete_package, packages)))

    LOGGER.info("📦 Preparing %s report", sbom_config.output_format)
    format_sbom(packages, relationships, sbom_config)


if __name__ == "__main__":
    scan(prog_name="sbom")

==> fluid_sbom-1.1.18.tar.gz: fluid_sbom-1.1.18/fluid_sbom/cli/__init__.py <==
import asyncio
import logging
import os
import tempfile
import textwrap
from concurrent.futures import (
    ThreadPoolExecutor,
)
from typing import (
    Unpack,
)

import click

from fluid_sbom.advisories.database import (
    DATABASE,
)
from fluid_sbom.cli.options import (
    MutuallyExclusiveOption,
    RequiredAnyCommand,
)
from fluid_sbom.cli.sbom_config import (
    ScanArgs,
    build_sbom_config,
)
from fluid_sbom.config.bugsnag import (
    initialize_bugsnag,
)
from fluid_sbom.config.logger import (
    configure_logger,
    modify_logger_level,
)
from fluid_sbom.format import (
    format_sbom,
)
from fluid_sbom.internal.file_resolver.container_image import (
    ContainerImage,
    ImageContext,
)
from fluid_sbom.model.core import (
    SourceType,
)
from fluid_sbom.pkg.cataloger.complete import (
    complete_package,
)
from fluid_sbom.pkg.operations.package_operation import (
    package_operations_factory,
)
from fluid_sbom.sources.directory_source import (
    Directory,
)
from fluid_sbom.sources.docker import (
    ImageMetadata,
    extract_docker_image,
    get_docker_image,
)
from fluid_sbom.sources.ecr import AwsRole, ecr_connection
from fluid_sbom.utils.exceptions import UnexpectedSBOMSourceError

LOGGER = logging.getLogger(__name__)


def show_banner() -> None:
    logo = textwrap.dedent(
        """
         ───── ⌝
        |    ⌝|  Fluid Attacks
        |  ⌝  |  We hack your software.
         ─────
        """,
    )
    click.secho(logo, fg="red")


def get_context(  # noqa: PLR0913
    image: ImageMetadata,
    *,
    username: str | None = None,
    password: str | None = None,
    token: str | None = None,
    aws_creds: str | None = None,
    daemon: bool = False,
) -> ImageContext | None:
    temp_dir = tempfile.mkdtemp()
    layers_dir, manifest = extract_docker_image(
        image,
        temp_dir,
        username=username,
        password=password,
        token=token,
        aws_creds=aws_creds,
        daemon=daemon,
    )

    return ImageContext(
        id=image.digest,
        name=image.name,
        publisher="",
        arch=image.architecture,
        size=str(sum(x.size for x in image.layersdata)),
        full_extraction_dir=temp_dir,
        layers_dir=layers_dir,
        manifest=manifest,
        image_ref=image.image_ref,
    )


@click.command(cls=RequiredAnyCommand, required_any=["o_from", "config"])
@click.argument("arg")
@click.option(
    "--from",
    "o_from",
    type=click.Choice(
        ["docker", "dir", "docker-daemon", "ecr"],
        case_sensitive=False,
    ),
    help=(
        "Source of the scan: 'docker' for scanning Docker images or 'dir' for scanning directories."
    ),
    cls=MutuallyExclusiveOption,
    mutually_exclusive=["config"],
)
@click.option(
    "--format",
    "output_format",
    type=click.Choice(
        [
            "fluid-json",
            "cyclonedx-json",
            "spdx-json",
            "cyclonedx-xml",
            "spdx-xml",
        ],
        case_sensitive=False,
    ),
    default="fluid-json",
    help="Output format for the scanned data.",
    cls=MutuallyExclusiveOption,
    mutually_exclusive=["config"],
)
@click.option(
    "--output",
    "-o",
    help="Output filename.",
    default="sbom",
    cls=MutuallyExclusiveOption,
    mutually_exclusive=["config"],
)
@click.option(
    "--docker-user",
    default=None,
    help="Docker registry username.",
    cls=MutuallyExclusiveOption,
    mutually_exclusive=["config"],
)
@click.option(
    "--docker-password",
    default=None,
    help="Docker registry password.",
    cls=MutuallyExclusiveOption,
    mutually_exclusive=["config"],
)
@click.option(
    "--aws-external-id",
    default=None,
    help="Docker registry username.",
    cls=MutuallyExclusiveOption,
    mutually_exclusive=["config"],
)
@click.option(
    "--aws-role",
    default=None,
    help="Docker registry password.",
    cls=MutuallyExclusiveOption,
    mutually_exclusive=["config"],
)
@click.option(
    "--config",
    "-c",
    is_flag=True,
    default=False,
    cls=MutuallyExclusiveOption,
    help="Path to an advanced configuration file with additional settings.",
    mutually_exclusive=[
        "aws-role",
        "aws-external-id",
        "docker-user",
        "docker-password",
        "output_format",
        "output",
        "o_from",
    ],
)
@click.option(
    "--debug",
    help="Run the application on debug mode",
    is_flag=True,
)
def scan(arg: str, **kwargs: Unpack[ScanArgs]) -> None:
    configure_logger()
    initialize_bugsnag()
    show_banner()

    sbom_config = build_sbom_config(arg, **kwargs)

    if sbom_config.debug or kwargs["debug"]:
        modify_logger_level()

    match sbom_config.source_type:
        case SourceType.DIRECTORY:
            sbom_config.resolver = Directory(
                root=sbom_config.source,
                exclude=sbom_config.exclude,
            )
        case SourceType.DOCKER | SourceType.DOCKER_DAEMON:
            daemon = sbom_config.source_type == SourceType.DOCKER_DAEMON
            docker_image = get_docker_image(
                sbom_config.source,
                username=sbom_config.docker_user,
                password=sbom_config.docker_password,
                daemon=daemon,
            )
            if not docker_image:
                error_msg = f"No image found for {sbom_config.source}"
                raise ValueError(error_msg)

            context = get_context(
                docker_image,
                username=sbom_config.docker_user,
                password=sbom_config.docker_password,
                daemon=daemon,
            )
            if context is None:
                error_msg = f"No context found for {docker_image}"
                raise ValueError(error_msg)
            sbom_config.resolver = ContainerImage(
                img=docker_image,
                context=context,
                lazy=False,
            )
        case SourceType.ECR:
            if not sbom_config.aws_role:
                error_msg = "The AWS role wasn't defined"
                raise ValueError(error_msg)
            role = AwsRole(
                external_id=sbom_config.aws_external_id,
                role=sbom_config.aws_role,
            )

            token, image_metadata = asyncio.run(
                ecr_connection(role, sbom_config.source),
            )

            if not image_metadata:
                error_msg = f"No image found for {sbom_config.source}"
                raise ValueError(error_msg)

            context = get_context(
                image_metadata,
                aws_creds=f"AWS:{token}",
            )
            if context is None:
                error_msg = f"No context found for {image_metadata}"
                raise ValueError(error_msg)
            sbom_config.resolver = ContainerImage(
                img=image_metadata,
                context=context,
                lazy=False,
            )
        case _:
            raise UnexpectedSBOMSourceError

    DATABASE.initialize()
    LOGGER.info(
        "📦 Generating SBOM from %s: %s",
        sbom_config.source_type.value,
        sbom_config.source,
    )

    packages, relationships = package_operations_factory(sbom_config.resolver)
    with ThreadPoolExecutor(
        max_workers=min(
            32,
            (os.cpu_count() or 1) * 5 if os.cpu_count() is not None else 32,
        ),
    ) as executor:
        LOGGER.info("📦 Gathering additional package information")
        packages = list(filter(None, executor.map(complete_package, packages)))

    LOGGER.info("📦 Preparing %s report", sbom_config.output_format)
    format_sbom(packages, relationships, sbom_config)


if __name__ == "__main__":
    scan(prog_name="sbom")
